
# q-expansion of newform 704.4.c.b, downloaded from the LMFDB on 30 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 704
weight = 4
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 0, 1], 1], [[-5, 0, 10, 0], 1], [[0, 10, 0, -5], 1]]

hecke_ring_character_values = [[639, [1, 0, 0, 0]], [133, [-1, 0, 0, 0]], [321, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 5, 0, 0], [0, 0, 1, 0], [0, 0, 0, 2], [0, 11, 0, 0], [0, 0, 4, 0], [30, 0, 0, 0], [0, 44, 0, 0], [0, 0, 0, -7], [0, 0, 4, 0], [0, 0, 0, 39], [0, 0, -35, 0], [72, 0, 0, 0], [0, 250, 0, 0], [0, 0, 0, 14], [0, 0, 16, 0], [0, 471, 0, 0], [0, 0, -48, 0], [0, 205, 0, 0], [0, 0, 0, -27], [-520, 0, 0, 0], [0, 0, 0, -102], [0, -210, 0, 0], [-849, 0, 0, 0], [-1055, 0, 0, 0], [0, 0, -186, 0], [0, 0, 0, 86], [0, 630, 0, 0], [0, 0, -18, 0], [-1425, 0, 0, 0], [0, 0, 0, 84], [0, -642, 0, 0], [255, 0, 0, 0], [0, 2126, 0, 0], [0, 0, -38, 0], [0, 0, 0, 298], [0, 0, -337, 0], [0, -100, 0, 0], [0, 0, 0, -300], [0, 0, -522, 0], [0, 741, 0, 0], [0, 0, 121, 0], [0, 0, 0, -489], [2840, 0, 0, 0], [0, 0, 398, 0], [0, 0, 0, 142], [0, 1468, 0, 0], [0, 0, 0, -51], [0, 3090, 0, 0], [0, 0, 199, 0], [2880, 0, 0, 0], [0, 0, 0, -214], [3128, 0, 0, 0], [0, -723, 0, 0], [-150, 0, 0, 0], [0, 0, 0, 674], [0, 0, 176, 0], [0, 0, 0, 496], [0, 0, 366, 0], [8358, 0, 0, 0], [0, -4720, 0, 0], [0, 0, -1100, 0], [0, -7640, 0, 0], [0, 0, 0, 854], [8905, 0, 0, 0], [0, 0, 77, 0], [0, 1283, 0, 0], [-2450, 0, 0, 0], [0, -10860, 0, 0], [0, 0, 274, 0], [-9525, 0, 0, 0], [0, 0, 0, -540], [0, 0, 0, 655], [0, 0, -330, 0], [0, -10391, 0, 0], [0, 0, 0, -183], [0, 0, -241, 0], [0, 0, -736, 0], [7902, 0, 0, 0], [14246, 0, 0, 0], [0, 11556, 0, 0], [0, 0, -192, 0], [0, 0, 0, 946], [8035, 0, 0, 0], [0, 0, 0, -1904], [0, -495, 0, 0], [-7461, 0, 0, 0], [7460, 0, 0, 0], [0, 0, -524, 0], [0, 0, 0, -317], [0, 4305, 0, 0], [0, 0, 0, -248], [0, 0, 0, 727], [0, 21372, 0, 0], [0, 12364, 0, 0], [0, 0, 0, 1698], [0, 0, 2175, 0], [237, 0, 0, 0], [0, -3380, 0, 0], [0, 0, -336, 0], [0, -160, 0, 0], [0, 0, 2726, 0], [0, 14760, 0, 0], [9444, 0, 0, 0], [0, -12112, 0, 0], [-8615, 0, 0, 0], [0, 11340, 0, 0], [-25560, 0, 0, 0], [0, 0, 0, 2102], [23102, 0, 0, 0], [0, 0, 0, 1746], [0, 0, 2204, 0], [-24030, 0, 0, 0], [0, -20131, 0, 0], [0, 0, 0, 1281], [-26703, 0, 0, 0], [0, -5785, 0, 0], [0, 0, 0, -1839], [0, 0, 983, 0], [0, -14454, 0, 0], [0, 0, 2501, 0], [-8870, 0, 0, 0], [0, 0, 1494, 0], [0, 1680, 0, 0], [0, -12253, 0, 0], [0, 0, -2510, 0], [0, 0, -2513, 0], [0, 0, 0, 2479], [0, 0, 0, 627], [0, 0, -4036, 0], [0, -13574, 0, 0], [0, 0, 0, 2452], [0, 0, 0, 695], [0, 0, -2396, 0], [28668, 0, 0, 0], [-17944, 0, 0, 0], [0, 0, -1936, 0], [0, -21340, 0, 0], [0, 0, -107, 0], [-32904, 0, 0, 0], [0, 31582, 0, 0], [0, 0, 4342, 0], [0, 0, 0, 3889], [0, -300, 0, 0], [0, 0, 841, 0], [0, 0, 0, -3173], [0, 0, 2122, 0], [-27300, 0, 0, 0], [0, -49229, 0, 0], [0, 0, 0, 4370], [0, 0, -1736, 0], [47133, 0, 0, 0], [0, -38380, 0, 0], [0, 0, 0, 4346], [0, -4700, 0, 0], [0, 0, 0, -5230], [0, 0, 0, -4910], [234, 0, 0, 0], [34540, 0, 0, 0], [0, 0, -5446, 0], [0, -17985, 0, 0], [43470, 0, 0, 0], [0, 0, 0, 336], [0, -39237, 0, 0], [-36405, 0, 0, 0], [0, 0, 0, -3385], [0, 0, 0, 5458], [0, 0, -2686, 0]]
